/***************************************************************************
 *   Copyright (C) 2007-2010 by Tobias Schwarz                             *
 *   mail@tobias-schwarz.net                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#ifndef KEMBOSSWINDOW_H
#define KEMBOSSWINDOW_H

#include <QtGui>
#include "settings.h"

class kembossWindow : public QMainWindow
// kembossWindow is the abstract base class of the kemboss main window and the class kprogramWindow, which is the abstract base class of all program windows
{
   Q_OBJECT

public:
   kembossWindow(QMainWindow *parent = 0);
   virtual ~kembossWindow() {};
   QString curFile;
   void updateRecentFileActions();

protected:
   void readSettings();
   void writeSettings();
   void createActions();
   void createRecentFiles();
   void assistant(const QString &page);
   QMenu *recentSettingsMenu;
   QAction *reportBugAct;
   QAction *aboutAct;
   enum { MaxRecentFiles = 4 };
   QAction *recentFileActs[MaxRecentFiles];
   QString strippedName(const QString &fullFileName) { return QFileInfo(fullFileName).fileName(); }
   kembossWindow *kembossParent;
   QGraphicsView *kpwOutputGraphicsView;
   QToolBar *fileToolBar;
   QToolBar *editToolBar;
   QToolBar *viewToolBar;
   QToolBar *graphToolBar;
   QToolBar *runToolBar;
   QToolBar *helpToolBar;
   QAction *viewFileToolbarAct;
   QAction *viewEditToolbarAct;
   QAction *viewViewToolbarAct;
   QAction *viewGraphToolbarAct;
   QAction *viewRunToolbarAct;
   QAction *viewHelpToolbarAct;

   int highlightInput;
   int highlightOutput;

private:
   virtual bool loadFile(const QString &fileName) = 0;
   virtual bool saveFile(const QString &fileName) = 0;
   virtual bool maybeSave() = 0;
   virtual void setCurrentFile(const QString &fileName) = 0;
   void showFileToolbar(const bool &show) { fileToolBar->setVisible(show); viewFileToolbarAct->setChecked(show); }
   void showEditToolbar(const bool &show) { editToolBar->setVisible(show); viewEditToolbarAct->setChecked(show); }
   void showViewToolbar(const bool &show) { viewToolBar->setVisible(show); viewViewToolbarAct->setChecked(show); }
   void showGraphToolbar(const bool &show) { graphToolBar->setVisible(show); viewGraphToolbarAct->setChecked(show); }
   void showRunToolbar(const bool &show) { runToolBar->setVisible(show); viewRunToolbarAct->setChecked(show); }
   void showHelpToolbar(const bool &show) { helpToolBar->setVisible(show); viewHelpToolbarAct->setChecked(show); }

protected slots:
   bool save();
   bool saveAs();
   void open();

private slots:
   void openRecentFile();
   void aboutKemboss();
   void viewFileToolbar() { showFileToolbar(!fileToolBar->isVisible()); } ;
   void viewEditToolbar() { showEditToolbar(!editToolBar->isVisible()); } ;
   void viewViewToolbar() { showViewToolbar(!viewToolBar->isVisible()); } ;
   void viewGraphToolbar() { showGraphToolbar(!graphToolBar->isVisible()); } ;
   void viewRunToolbar() { showRunToolbar(!runToolBar->isVisible()); } ;
   void viewHelpToolbar() { showHelpToolbar(!helpToolBar->isVisible()); } ;
   void reportBug() { QDesktopServices::openUrl (QUrl("mailto:kemboss@tobias-schwarz.net?subject=Bug in "+objectName())); };
};

#endif
