/***************************************************************************
 *   Copyright (C) 2007-2010 by Tobias Schwarz                             *
 *   mail@tobias-schwarz.net                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#ifndef WINDOWCLASSES_H
#define WINDOWCLASSES_H

#include <QtGui>
#include "programProgress.h"
#include "kembossWindowClass.h"
#include "syntax_highlighter.h"

class kprogramWindow : public kembossWindow
// kprogramWindow is the abstract base class of all program windows
{
   Q_OBJECT

public:
   kprogramWindow(kembossWindow *parent = 0);
   virtual ~kprogramWindow() {};
   bool loadFile(const QString &fileName);
   void loadFormats(const QString &formatFile, QWidget *widget);
   void setCurrentFile(const QString &fileName);
   virtual void setEmbossApp(const QString &appName) { if (!appName.isEmpty()) cembossApp = appName; };
   QString embossApp() { return cembossApp; };
   void selectOpenFile(const QString &msg, QLineEdit *lineEdit, bool load=false);
   void selectSaveFile(const QString &msg, QLineEdit *lineEdit, bool save=false);
   QStringList loadRebaseEnzymes();
   void saveEnzymeList(QListWidget *enzymeList);
   QString selectedEnzymes(QListWidget *enzymeList);
   QStringList programConfiguration;
   QStringList associatedQualifiers;
   QStringList additionalConfigurationParameters;
   QComboBox *kpwProgramComboBox;
   void setTextEditFont();

public slots:
   void openSAQForm(const QString num = "1");
   void openOAQForm(const QString num = "2");
   void openOsAQForm(const QString num = "2");
   void openOfAQForm(const QString num = "3");
   void openFAQForm(const QString num = "1");
   void openGAQForm(const QString num = "");
   void openAAQForm(const QString num = "3");

private slots:
   void newSettings();
   void addParameter();
   void copyCommandline();
   bool runEmbossApp();
   void printDocument();
   void documentProperties();
   void zoomIn();
   void zoomOut();
   void zoomOriginal();
   void rotateLeft();
   void rotateRight();
   void highlightNone();
   void highlightFastaAA();
   void highlightFastaNuc();
   void highlightAlignmentAA();
   void highlightAlignmentNuc();
   void highlightEmbl();
   void highlightGbAA();
   void highlightGbNuc();
   void highlightingChanges(int index);
   void updateProgramProgressForm();
   void processFinished(int exitCode);
   void killProcess();
   void embossHelp();
   bool on_OpenInputFilePushButton_clicked();
   bool on_SaveInputFileAsPushButton_clicked();
   bool on_SaveOutputFileAsPushButton_clicked();
   //void appendText(QString text, bool finished);
   void tabChanged(const int &index);
   //void viewTab();
   void wasModified() { setWindowModified(true); saveAct->setEnabled(true); }

private:
   void createActions();
   void createMenus();
   void createToolbars();
   void findControls();
   bool saveFile(const QString &fileName);
   void deleteTemporaryFiles();
   bool saveOutputFile(const QString &fileName);
   bool maybeSave();
   bool maybeSaveInputFile();
   bool maybeSaveOutputFile();
   bool createProgramConfiguration();
   void loadOutputGraphics();
   bool checkInput();
   bool checkOutput();
   bool checkEmbossApp();
   void loadPrinterSettings();
   void savePrinterSettings();
   QProcess programProcess;
   programProgressForm *programProgress;
   QTime runTime;
   QString tempInputFile;
   QString tempOutputFile;
   QStringList tempFiles;
   QString cembossApp;
   QString defaultSettingsFile();
   QPrinter printer;
   qreal graphScale;
//   loadFileThread loadThread;
//   QTextEdit *activeTextEdit;

   QMenu *fileMenu;
   QMenu *editMenu;
   QMenu *graphMenu;
   QMenu *viewMenu;
   QMenu *syntaxHighlightingMenu;
   QMenu *toolbarsMenu;
   //QMenu *tabsMenu;
   QMenu *runMenu;
   QMenu *helpMenu;

   QAction *newAct;
   QAction *openAct;
   QAction *saveAct;
   QAction *saveAsAct;
   QAction *exitAct;
   QAction *runAct;
   QAction *addParameterAct;
   QAction *copyCommandlineAct;
   QAction *cutAct;
   QAction *copyAct;
   QAction *pasteAct;
   QAction *undoAct;
   QAction *redoAct;
   QAction *helpAct;
   QAction *printAct;
   QAction *propertiesAct;
   QAction *zoomInAct;
   QAction *zoomOutAct;
   QAction *zoomOrigAct;
   QAction *rotateLeftAct;
   QAction *rotateRightAct;
   QAction *highlightNoneAct;
   QAction *highlightFastaAAAct;
   QAction *highlightFastaNucAct;
   QAction *highlightAlignmentAAAct;
   QAction *highlightAlignmentNucAct;
   QAction *highlightEmblAct;
   QAction *highlightGbAAAct;
   QAction *highlightGbNucAct;

   QTextEdit *kpwInputTextEdit;
   QLineEdit *kpwInputFileLineEdit;
   QLineEdit *kpwOutputFileLineEdit;
   QTextEdit *kpwOutputTextEdit;
   QTabWidget *kpwMainTab;

   Highlighter *highlighter;
   QComboBox *selectHighlighting;

protected:
   bool loadInputFile(const QString &fileName);
   bool saveInputFile(const QString &fileName);
   void postWindowCreated();
   void closeEvent(QCloseEvent *event);
   virtual void beforeRunningProgram();
   virtual void afterRunningProgram();
   virtual void completeOutputFiles(QLineEdit *lineEdit, const QString &extension);
};

#endif
